@@ -19,15 +19,18 @@
 import gtk, gtk.glade, gmenu, gobject, gnomevfs, gnome.ui
 import cgi, os
 import gettext
-gettext.bindtextdomain('alacarte')
-gettext.textdomain('alacarte')
-gtk.glade.bindtextdomain('alacarte')
+import subprocess
+try:
+	from Alacarte import config
+	gettext.bindtextdomain('alacarte',config.localedir)
+	gettext.textdomain('alacarte')
+	gtk.glade.bindtextdomain('alacarte',config.localedir)
+except:
+	pass
 gtk.glade.textdomain('alacarte')
 _ = gettext.gettext
 from Alacarte.MenuEditor import MenuEditor
-from Alacarte.DialogHandler import DialogHandler
 from Alacarte import util
-import time
 
 class MainWindow:
 	timer = None
@@ -38,11 +41,13 @@ class MainWindow:
 	dnd_menus = [('ALACARTE_MENU_ROW', gtk.TARGET_SAME_APP, 0)]
 	dnd_both = [dnd_items[0],] + dnd_menus
 	drag_data = None
+	edit_pool = []
 
 	def __init__(self, datadir, version, argv):
 		self.file_path = datadir
 		self.version = version
 		self.editor = MenuEditor()
+		self.gnome_program = gnome.init('alacarte', version)
 		gtk.window_set_default_icon_name('alacarte')
 		self.tree = gtk.glade.XML(os.path.join(self.file_path, 'alacarte.glade'))
 		signals = {}
@@ -51,7 +56,6 @@ class MainWindow:
 		self.tree.signal_autoconnect(signals)
 		self.setupMenuTree()
 		self.setupItemTree()
-		self.dialogs = DialogHandler(self.tree.get_widget('mainwindow'), self.editor, self.file_path)
 		self.tree.get_widget('edit_delete').set_sensitive(False)
 		self.tree.get_widget('edit_revert_to_original').set_sensitive(False)
 		self.tree.get_widget('edit_properties').set_sensitive(False)
@@ -62,13 +66,15 @@ class MainWindow:
 		accelgroup.connect_group(keyval, modifier, gtk.ACCEL_VISIBLE, self.on_mainwindow_undo)
 		keyval, modifier = gtk.accelerator_parse('<Ctrl><Shift>Z')
 		accelgroup.connect_group(keyval, modifier, gtk.ACCEL_VISIBLE, self.on_mainwindow_redo)
+		keyval, modifier = gtk.accelerator_parse('F1')
+		accelgroup.connect_group(keyval, modifier, gtk.ACCEL_VISIBLE, self.on_help_button_clicked)
 		self.tree.get_widget('mainwindow').add_accel_group(accelgroup)
 		gnome.ui.authentication_manager_init()
 
 	def run(self):
 		self.loadMenus()
-		self.editor.applications.tree.add_monitor(self.menuChanged)
-		self.editor.settings.tree.add_monitor(self.menuChanged)
+		self.editor.applications.tree.add_monitor(self.menuChanged, None)
+		self.editor.settings.tree.add_monitor(self.menuChanged, None)
 		self.tree.get_widget('mainwindow').show_all()
 		gtk.main()
 
@@ -120,8 +126,9 @@ class MainWindow:
 				found = False
 				if item[3].get_type() == gmenu.TYPE_ENTRY and item[3].get_desktop_file_id() == item_id:
 					found = True
-				if item[3].get_type() == gmenu.TYPE_DIRECTORY and os.path.split(item[3].get_desktop_file_path())[1] == item_id:
-					found = True
+				if item[3].get_type() == gmenu.TYPE_DIRECTORY and item[3].get_desktop_file_path():
+					if os.path.split(item[3].get_desktop_file_path())[1] == item_id:
+						found = True
 				if item[3].get_type() == gmenu.TYPE_SEPARATOR:
 					if not isinstance(item_id, tuple):
 						continue
@@ -210,7 +217,11 @@ class MainWindow:
 		menu_tree = self.tree.get_widget('menu_tree')
 		menu_tree.set_model(self.menu_store)
 		for menu in self.menu_store:
-			menu_tree.expand_to_path(menu.path)
+			#this might not work for some reason
+			try:
+				menu_tree.expand_to_path(menu.path)
+			except:
+				pass
 		menu_tree.get_selection().select_path((0,))
 		self.on_menu_tree_cursor_changed(menu_tree)
 
@@ -244,6 +255,33 @@ class MainWindow:
 				icon = util.getIcon(item)
 			self.item_store.append((show, icon, name, item))
 
+	#this is a little timeout callback to insert new items after
+	#gnome-desktop-item-edit has finished running
+	def waitForNewItemProcess(self, process, parent_id, file_path):
+		if process.poll() != None:
+			if os.path.isfile(file_path):
+				self.editor.insertExternalItem(os.path.split(file_path)[1], parent_id)
+			return False
+		return True
+
+	def waitForNewMenuProcess(self, process, parent_id, file_path):
+		if process.poll() != None:
+			#hack for broken gnome-desktop-item-edit
+			broken_path = os.path.join(os.path.split(file_path)[0], '.directory')
+			if os.path.isfile(broken_path):
+				os.rename(broken_path, file_path)
+			if os.path.isfile(file_path):
+				self.editor.insertExternalMenu(os.path.split(file_path)[1], parent_id)
+			return False
+		return True
+
+	#this callback keeps you from editing the same item twice
+	def waitForEditProcess(self, process, file_path):
+		if process.poll() != None:
+			self.edit_pool.remove(file_path)
+			return False
+		return True
+
 	def on_new_menu_button_clicked(self, button):
 		menu_tree = self.tree.get_widget('menu_tree')
 		menus, iter = menu_tree.get_selection().get_selected()
@@ -253,11 +291,9 @@ class MainWindow:
 			menu_tree.get_selection().select_path((0,))
 		else:
 			parent = menus[iter][2]
-		values = self.dialogs.newMenuDialog()
-		if values:
-			self.editor.createMenu(parent, values[0], values[1], values[2])
-		#FIXME: make gnome-menus update monitors when menus are added
-		gobject.timeout_add(3, self.loadUpdates)
+		file_path = os.path.join(util.getUserDirectoryPath(), util.getUniqueFileId('alacarte-made', '.directory'))
+		process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
+		gobject.timeout_add(100, self.waitForNewMenuProcess, process, parent.menu_id, file_path)
 
 	def on_new_item_button_clicked(self, button):
 		menu_tree = self.tree.get_widget('menu_tree')
@@ -268,9 +304,9 @@ class MainWindow:
 			menu_tree.get_selection().select_path((0,))
 		else:
 			parent = menus[iter][2]
-		values = self.dialogs.newItemDialog()
-		if values:
-			self.editor.createItem(parent, values[0], values[1], values[2], values[3], values[4])
+		file_path = os.path.join(util.getUserItemPath(), util.getUniqueFileId('alacarte-made', '.desktop'))
+		process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
+		gobject.timeout_add(100, self.waitForNewItemProcess, process, parent.menu_id, file_path)
 
 	def on_new_separator_button_clicked(self, button):
 		item_tree = self.tree.get_widget('item_tree')
@@ -314,13 +350,26 @@ class MainWindow:
 		if not iter:
 			return
 		item = items[iter][3]
-		self.allow_update = False
+		if item.get_type() not in (gmenu.TYPE_ENTRY, gmenu.TYPE_DIRECTORY):
+			return
+
 		if item.get_type() == gmenu.TYPE_ENTRY:
-			self.dialogs.editItemDialog(items[iter])
+			file_path = os.path.join(util.getUserItemPath(), item.get_desktop_file_id())
+			file_type = 'Item'
 		elif item.get_type() == gmenu.TYPE_DIRECTORY:
-			self.dialogs.editMenuDialog(items[iter])
-		self.allow_update = True
-		self.loadUpdates()
+			file_path = os.path.join(util.getUserDirectoryPath(), os.path.split(item.get_desktop_file_path())[1])
+			file_type = 'Menu'
+
+		if not os.path.isfile(file_path):
+			data = open(item.get_desktop_file_path()).read()
+			open(file_path, 'w').write(data)
+			self.editor._MenuEditor__addUndo([(file_type, os.path.split(file_path)[1]),])
+		else:
+			self.editor._MenuEditor__addUndo([item,])
+		if file_path not in self.edit_pool:
+			self.edit_pool.append(file_path)
+			process = subprocess.Popen(['gnome-desktop-item-edit', file_path], env=os.environ)
+			gobject.timeout_add(100, self.waitForEditProcess, process, file_path)
 
 	def on_menu_tree_cursor_changed(self, treeview):
 		menus, iter = treeview.get_selection().get_selected()
@@ -368,6 +417,7 @@ class MainWindow:
 			self.editor.setVisible(item, False)
 		else:
 			self.editor.setVisible(item, True)
+		self.item_store[path][0] = not self.item_store[path][0]
 
 	def on_item_tree_cursor_changed(self, treeview):
 		items, iter = treeview.get_selection().get_selected()
@@ -381,8 +431,18 @@ class MainWindow:
 			self.tree.get_widget('edit_properties').set_sensitive(True)
 		else:
 			self.tree.get_widget('edit_properties').set_sensitive(False)
-		self.tree.get_widget('move_up_button').set_sensitive(True)
-		self.tree.get_widget('move_down_button').set_sensitive(True)
+
+		# If first item...
+		if items.get_path(iter)[0] == 0:
+			self.tree.get_widget('move_up_button').set_sensitive(False)
+		else:
+			self.tree.get_widget('move_up_button').set_sensitive(True)
+
+		# If last item...
+		if items.get_path(iter)[0] == (len(items)-1):
+			self.tree.get_widget('move_down_button').set_sensitive(False)
+		else:
+			self.tree.get_widget('move_down_button').set_sensitive(True)
 
 	def on_item_tree_row_activated(self, treeview, path, column):
 		self.on_edit_properties_activate(None)
@@ -518,6 +578,9 @@ class MainWindow:
 	def on_mainwindow_redo(self, accelgroup, window, keyval, modifier):
 		self.editor.redo()
 
+	def on_help_button_clicked(self, *args):
+		gnome.help_display_desktop(self.gnome_program, 'user-guide', 'user-guide.xml', 'menu-editor')
+
 	def on_revert_button_clicked(self, button):
 		dialog = self.tree.get_widget('revertdialog')
 		dialog.set_transient_for(self.tree.get_widget('mainwindow'))
